const merge = require('merge2');
const sourcemaps = require('gulp-sourcemaps');
const ts = require('gulp-typescript');

const ng2InlineTemplate = require('./ng2-inline-template').ng2InlineTemplate;

const tsProject = ts.createProject('./tsconfig.json');

const getTsSource = (src) => {
  return [src, '!node_modules/**/*.ts', 'node_modules/@types/**/*.d.ts'];
};  

const buildFactory = (src, dest, isAngular, hasDts) => {
  let result;

  let tsResult = gulp.src(getTsSource(src))
    .pipe(sourcemaps.init());

  if (isAngular){
    tsResult = tsResult.pipe(ng2InlineTemplate());
  }

  tsResult = tsResult.pipe(tsProject());

  if (hasDts){
    result = merge([
        tsResult.dts.pipe(gulp.dest(dest)),
        tsResult.js.pipe(sourcemaps.write())
          .pipe(gulp.dest(dest))
      ]);
  } else {
    result = tsResult
      .js.pipe(sourcemaps.write())
      .pipe(gulp.dest(dest))
  }

  return result;
};

const buildServer = (src, dest) => {
  return buildFactory(src, dest, false, false); 
};

const buildLibrary = (src, dest) => {
  return buildFactory(src, dest, true, true);
};

const buildProject = (src, dest) => {
  return buildFactory(src, dest, true, false);
};

exports.buildLibrary = buildLibrary;
exports.buildProject = buildProject;
exports.buildServer = buildServer;